import { computed, ref, unref } from 'vue'
import { debounce } from '@cip/utils/util'
/**
 * 分页hook
 * @param initOffset
 * @param initLimit
 * @return {{total: Ref<UnwrapRef<number>>, offset: Ref<UnwrapRef<number>>, limit: Ref<UnwrapRef<number>>, currentPage: Ref<UnwrapRef<number>>}}
 */
export const usePaging = ({ offset: initOffset = 0, limit: initLimit = 10 }) => {
  const offset = ref(initOffset)
  const limit = ref(initLimit)
  const total = ref(0)
  const currentPage = ref(1)
  return {
    offset,
    limit,
    total,
    currentPage
  }
}

/**
 * 列表页hook
 * @param Entity
 * @param itemType
 * @param pageFn
 * @param infoFn
 * @param limit
 * @returns {{getItemList: (function(): *), itemType: *, item: Ref<UnwrapRef<{}>>, showOnly: Ref<UnwrapRef<boolean>>, offset: RefTypes.Ref<UnwrapRef<number>>, itemDialog: Ref<UnwrapRef<boolean>>, getItem: (function(*): *), dialogType: Ref<UnwrapRef<string>>, total: RefTypes.Ref<UnwrapRef<number>>, search: search, defaultSearchFilter: Ref<UnwrapRef<{}>>, listLoading: Ref<UnwrapRef<boolean>>, limit: RefTypes.Ref<UnwrapRef<number>>, showItem: ((function(*, *=): Promise<void>)|*), itemList: Ref<UnwrapRef<*[]>>, dialogTitle: ComputedRef<string>, currentPage: RefTypes.Ref<UnwrapRef<number>>, searchFilter: Ref<UnwrapRef<{}>>}}
 */
export const useRetrieve = (Entity, { itemType, pageFn = 'page', infoFn = 'info', limit: initLimit = 10 } = { }) => {
  const {
    offset,
    limit,
    total,
    currentPage
  } = usePaging({ offset: 0, limit: initLimit })

  const listLoading = ref(false)
  const itemList = ref([])
  const item = ref({})
  const showOnly = ref(false)
  const itemDialog = ref(false)
  const dialogType = ref('')
  const searchFilter = ref({})
  const defaultSearchFilter = ref({})
  const dialogTitle = computed(() => {
    return `${dialogType.value}${itemType}`
  })

  const getItemList = debounce(async (params = {}) => {
    try {
      listLoading.value = true
      const res = await unref(Entity)[unref(pageFn)]({
        ...defaultSearchFilter.value, // 默认搜索条件
        ...searchFilter.value, // 搜索条件
        ...params // 调用是传入的雕件
      }, { offset: offset.value, limit: limit.value })
      itemList.value = res.data || []
      offset.value = res.offset || 0
      total.value = res.total || 0
      currentPage.value = res.pageNum || 1
    } finally {
      listLoading.value = false
    }
  }, 50, false)

  const getItem = (row) => {
    return unref(Entity)[unref(infoFn)](row)
  }
  const showItem = async (row, fetch = false) => {
    try {
      if (fetch) {
        const { data } = await getItem(row)
        item.value = data
      } else {
        item.value = row // 查看时不需要深克隆，此处不应该对row进行任何的修改
      }
      dialogType.value = '查看'
      showOnly.value = true
      itemDialog.value = true
    } catch (err) {}
  }
  const search = () => {
    offset.value = 0
    getItemList()
  }
  return {
    itemType,
    listLoading,
    itemList,
    item,
    itemDialog,
    dialogTitle,
    dialogType,
    showOnly,
    offset,
    limit,
    total,
    currentPage,
    searchFilter,
    defaultSearchFilter,
    getItemList,
    getItem,
    showItem,
    search
  }
}
